@@ -7,7 +7,7 @@ package IO::Async::SignalProxy;
 
 use strict;
 
-our $VERSION = '0.14';
+our $VERSION = '0.15';
 
 use base qw( IO::Async::Notifier );
 
@@ -137,6 +137,12 @@ sub attach
    my $self = shift;
    my ( $signal, $code ) = @_;
 
+   if( $signal eq "CHLD" ) {
+      # We make special exception to allow the ChildManager to do this
+      caller(1) eq "IO::Async::ChildManager" or
+         carp "Attaching to SIGCHLD is not advised - use the IO::Async::ChildManager instead";
+   }
+
    exists $SIG{$signal} or croak "Unrecognised signal name $signal";
 
    # Don't allow anyone to trash an existing signal handler
